@@ -1,9 +1,8 @@
-# encoding: utf-8
 # No docstring, so we can test D100
 from functools import wraps
 import os
-import sys
 from .expected import Expectation
+from typing import overload
 
 
 expectation = Expectation()
@@ -20,12 +19,29 @@ class class_:
         """"""
 
     @expect('D102: Missing docstring in public method')
-    def method():
+    def method(self=None):
         pass
 
-    def _ok_since_private():
+    def _ok_since_private(self=None):
         pass
 
+    @overload
+    def overloaded_method(self, a: int) -> str:
+        ...
+
+    @overload
+    def overloaded_method(self, a: str) -> str:
+        """Foo bar documentation."""
+        ...
+
+    def overloaded_method(a):
+        """Foo bar documentation."""
+        return str(a)
+
+    expect('overloaded_method',
+           "D418: Function/ Method decorated with @overload"
+           " shouldn't contain a docstring")
+
     @expect('D102: Missing docstring in public method')
     def __new__(self=None):
         pass
@@ -54,6 +70,48 @@ def function():
         ''
 
 
+def function_with_nesting():
+    """Foo bar documentation."""
+    @overload
+    def nested_overloaded_func(a: int) -> str:
+        ...
+
+    @overload
+    def nested_overloaded_func(a: str) -> str:
+        """Foo bar documentation."""
+        ...
+
+    def nested_overloaded_func(a):
+        """Foo bar documentation."""
+        return str(a)
+
+
+expect('nested_overloaded_func',
+       "D418: Function/ Method decorated with @overload"
+       " shouldn't contain a docstring")
+
+
+@overload
+def overloaded_func(a: int) -> str:
+    ...
+
+
+@overload
+def overloaded_func(a: str) -> str:
+    """Foo bar documentation."""
+    ...
+
+
+def overloaded_func(a):
+    """Foo bar documentation."""
+    return str(a)
+
+
+expect('overloaded_func',
+       "D418: Function/ Method decorated with @overload"
+       " shouldn't contain a docstring")
+
+
 @expect('D200: One-line docstring should fit on one line with quotes '
         '(found 3)')
 @expect('D212: Multi-line docstring summary should start at the first line')
@@ -228,7 +286,7 @@ def around():
 @expect('D210: No whitespaces allowed surrounding docstring text')
 @expect('D213: Multi-line docstring summary should start at the second line')
 def multiline():
-    """ Whitespace at the begining.
+    """ Whitespace at the beginning.
 
     This is the end.
     """
@@ -270,23 +328,25 @@ def double_quotes_backslash_uppercase():
     R"""Sum\\mary."""
 
 
-if sys.version_info[0] <= 2:
-    @expect('D302: Use u""" for Unicode docstrings')
-    def unicode_unmarked():
-        """Юникод."""
+@expect('D213: Multi-line docstring summary should start at the second line')
+def exceptions_of_D301():
+    """Exclude some backslashes from D301.
 
-    @expect('D302: Use u""" for Unicode docstrings')
-    def first_word_has_unicode_byte():
-        """あy."""
+    In particular, line continuations \
+    and unicode literals \u0394 and \N{GREEK CAPITAL LETTER DELTA}.
+    They are considered to be intentionally unescaped.
+    """
 
 
 @expect("D400: First line should end with a period (not 'y')")
+@expect("D415: First line should end with a period, question mark, "
+        "or exclamation point (not 'y')")
 def lwnlkjl():
     """Summary"""
 
 
-@expect("D401: First line should be in imperative mood ('Return', not "
-        "'Returns')")
+@expect("D401: First line should be in imperative mood "
+        "(perhaps 'Return', not 'Returns')")
 def liouiwnlkjl():
     """Returns foo."""
 
@@ -297,6 +357,12 @@ def sdgfsdg23245():
     """Constructor for a foo."""
 
 
+@expect("D401: First line should be in imperative mood; try rephrasing "
+        "(found 'Constructor')")
+def sdgfsdg23245777():
+    """Constructor."""
+
+
 @expect('D402: First line should not be the function\'s "signature"')
 def foobar():
     """Signature: foobar()."""
@@ -325,9 +391,38 @@ def oneliner_d102(): return
 
 
 @expect("D400: First line should end with a period (not 'r')")
+@expect("D415: First line should end with a period, question mark,"
+        " or exclamation point (not 'r')")
 def oneliner_withdoc(): """One liner"""
 
 
+def ignored_decorator(func):   # noqa: D400,D401,D415
+    """Runs something"""
+    func()
+    pass
+
+
+def decorator_for_test(func):   # noqa: D400,D401,D415
+    """Runs something"""
+    func()
+    pass
+
+
+@ignored_decorator
+def oneliner_ignored_decorator(): """One liner"""
+
+
+@decorator_for_test
+@expect("D400: First line should end with a period (not 'r')")
+@expect("D415: First line should end with a period, question mark,"
+        " or exclamation point (not 'r')")
+def oneliner_with_decorator_expecting_errors(): """One liner"""
+
+
+@decorator_for_test
+def valid_oneliner_with_decorator(): """One liner."""
+
+
 @expect("D207: Docstring is under-indented")
 @expect('D213: Multi-line docstring summary should start at the second line')
 def docstring_start_in_same_line(): """First Line.
@@ -357,7 +452,10 @@ def outer_function():
 
 
 @expect("D400: First line should end with a period (not 'g')")
-@expect("D401: First line should be in imperative mood ('Run', not 'Runs')")
+@expect("D401: First line should be in imperative mood "
+        "(perhaps 'Run', not 'Runs')")
+@expect("D415: First line should end with a period, question mark, "
+        "or exclamation point (not 'g')")
 def docstring_bad():
     """Runs something"""
     pass
@@ -368,22 +466,64 @@ def docstring_bad_ignore_all():  # noqa
     pass
 
 
-def docstring_bad_ignore_one():  # noqa: D400,D401
+def docstring_bad_ignore_one():  # noqa: D400,D401,D415
     """Runs something"""
     pass
 
 
-@expect("D401: First line should be in imperative mood ('Run', not 'Runs')")
-def docstring_ignore_violations_of_pydocstyle_D400_and_PEP8_E501_but_catch_D401():  # noqa: E501,D400
+@expect("D401: First line should be in imperative mood "
+        "(perhaps 'Run', not 'Runs')")
+def docstring_ignore_some_violations_but_catch_D401():  # noqa: E501,D400,D415
     """Runs something"""
     pass
 
 
+@expect(
+    "D401: First line should be in imperative mood "
+    "(perhaps 'Initiate', not 'Initiates')"
+)
+def docstring_initiates():
+    """Initiates the process."""
+
+
+@expect(
+    "D401: First line should be in imperative mood "
+    "(perhaps 'Initialize', not 'Initializes')"
+)
+def docstring_initializes():
+    """Initializes the process."""
+
+
 @wraps(docstring_bad_ignore_one)
 def bad_decorated_function():
     """Bad (E501) but decorated"""
     pass
 
 
+def valid_google_string():  # noqa: D400
+    """Test a valid something!"""
+
+
+@expect("D415: First line should end with a period, question mark, "
+        "or exclamation point (not 'g')")
+def bad_google_string():  # noqa: D400
+    """Test a valid something"""
+
+
+# This is reproducing a bug where AttributeError is raised when parsing class
+# parameters as functions for Google / Numpy conventions.
+class Blah:  # noqa: D203,D213
+    """A Blah.
+
+    Parameters
+    ----------
+    x : int
+
+    """
+
+    def __init__(self, x):
+        pass
+
+
 expect(os.path.normcase(__file__ if __file__[-1] != 'c' else __file__[:-1]),
        'D100: Missing docstring in public module')
