@@ -19,10 +19,14 @@ from pydocstyle.checker import check
     'superfluous_quotes',
     'noqa',
     'sections',
+    'functions',
+    'canonical_google_examples',
+    'canonical_numpy_examples',
+    'canonical_pep257_examples',
 ])
 def test_complex_file(test_case):
     """Run domain-specific tests from test.py file."""
-    case_module = __import__('test_cases.{}'.format(test_case),
+    case_module = __import__(f'test_cases.{test_case}',
                              globals=globals(),
                              locals=locals(),
                              fromlist=['expectation'],
@@ -33,8 +37,9 @@ def test_complex_file(test_case):
                                   test_case + '.py')
     results = list(check([test_case_file],
                          select=set(ErrorRegistry.get_error_codes()),
-                         ignore_decorators=re.compile('wraps')))
+                         ignore_decorators=re.compile(
+                             'wraps|ignored_decorator')))
     for error in results:
         assert isinstance(error, Error)
-    results = set([(e.definition.name, e.message) for e in results])
+    results = {(e.definition.name, e.message) for e in results}
     assert case_module.expectation.expected == results
