@@ -1,13 +1,8 @@
-# -*- coding: utf-8 -*-
-
 """Use tox or py.test to run the test-suite."""
 
-from __future__ import with_statement
 from collections import namedtuple
 
 import os
-import sys
-import mock
 import shlex
 import shutil
 import pytest
@@ -15,6 +10,9 @@ import pathlib
 import tempfile
 import textwrap
 import subprocess
+import sys
+
+from unittest import mock
 
 from pydocstyle import checker, violations
 
@@ -22,7 +20,7 @@ from pydocstyle import checker, violations
 __all__ = ()
 
 
-class SandboxEnv(object):
+class SandboxEnv:
     """An isolated environment where pydocstyle can be run.
 
     Since running pydocstyle as a script is affected by local config files,
@@ -34,12 +32,19 @@ class SandboxEnv(object):
 
     Result = namedtuple('Result', ('out', 'err', 'code'))
 
-    def __init__(self, script_name='pydocstyle'):
+    def __init__(
+        self,
+        script_name='pydocstyle',
+        section_name='pydocstyle',
+        config_name='tox.ini',
+    ):
         """Initialize the object."""
         self.tempdir = None
         self.script_name = script_name
+        self.section_name = section_name
+        self.config_name = config_name
 
-    def write_config(self, prefix='', name='tox.ini', **kwargs):
+    def write_config(self, prefix='', name=None, **kwargs):
         """Change an environment config file.
 
         Applies changes to `tox.ini` relative to `tempdir/prefix`.
@@ -50,10 +55,24 @@ class SandboxEnv(object):
         if not os.path.isdir(base):
             self.makedirs(base)
 
+        name = self.config_name if name is None else name
+        if name.endswith('.toml'):
+            def convert_value(val):
+                return (
+                    repr(val).lower()
+                    if isinstance(val, bool)
+                    else repr(val)
+                )
+        else:
+            def convert_value(val):
+                return val
+
         with open(os.path.join(base, name), 'wt') as conf:
-            conf.write("[{}]\n".format(self.script_name))
+            conf.write(f"[{self.section_name}]\n")
             for k, v in kwargs.items():
-                conf.write("{} = {}\n".format(k.replace('_', '-'), v))
+                conf.write("{} = {}\n".format(
+                    k.replace('_', '-'), convert_value(v)
+                ))
 
     def open(self, path, *args, **kwargs):
         """Open a file in the environment.
@@ -110,15 +129,29 @@ def install_package(request):
     script.
     """
     cwd = os.path.join(os.path.dirname(__file__), '..', '..')
-    subprocess.check_call(shlex.split("pip install -e ."), cwd=cwd)
+    subprocess.check_call(
+        [sys.executable, "-m", "pip", "install", "-e", "."], cwd=cwd
+    )
     yield
-    subprocess.check_call(shlex.split("pip uninstall ."), cwd=cwd)
+    subprocess.check_call(
+        [sys.executable, "-m", "pip", "uninstall", "-y", "pydocstyle"], cwd=cwd
+    )
 
 
-@pytest.yield_fixture(scope="function")
+@pytest.yield_fixture(scope="function", params=['ini', 'toml'])
 def env(request):
-    """Add an testing environment to a test method."""
-    with SandboxEnv() as test_env:
+    """Add a testing environment to a test method."""
+    sandbox_settings = {
+        'ini': {
+            'section_name': 'pydocstyle',
+            'config_name': 'tox.ini',
+        },
+        'toml': {
+            'section_name': 'tool.pydocstyle',
+            'config_name': 'pyproject.toml',
+        },
+    }[request.param]
+    with SandboxEnv(**sandbox_settings) as test_env:
         yield test_env
 
 
@@ -149,10 +182,9 @@ def parse_errors(err):
 def test_pep257_conformance():
     """Test that we conform to PEP 257."""
     base_dir = (pathlib.Path(__file__).parent / '..').resolve()
-    src_dirs = (base_dir, base_dir / 'tests')
-    src_files = []
-    for src_dir in src_dirs:
-        src_files.extend(str(path) for path in src_dir.glob('*.py'))
+    excluded = base_dir / 'tests' / 'test_cases'
+    src_files = (str(path) for path in base_dir.glob('**/*.py')
+                 if excluded not in path.parents)
 
     ignored = {'D104', 'D105'}
     select = violations.conventions.pep257 - ignored
@@ -169,25 +201,57 @@ def test_ignore_list():
             return foo
     ''')
     expected_error_codes = {'D100', 'D400', 'D401', 'D205', 'D209', 'D210',
-                            'D403'}
+                            'D403', 'D415', 'D213'}
     mock_open = mock.mock_open(read_data=function_to_check)
     from pydocstyle import checker
     with mock.patch.object(
-            checker, 'tokenize_open', mock_open, create=True):
-        errors = tuple(checker.check(['filepath']))
+            checker.tk, 'open', mock_open, create=True):
+        # Passing a blank ignore here explicitly otherwise
+        # checkers takes the pep257 ignores by default.
+        errors = tuple(checker.check(['filepath'], ignore={}))
         error_codes = {error.code for error in errors}
         assert error_codes == expected_error_codes
 
     # We need to recreate the mock, otherwise the read file is empty
     mock_open = mock.mock_open(read_data=function_to_check)
     with mock.patch.object(
-            checker, 'tokenize_open', mock_open, create=True):
+            checker.tk, 'open', mock_open, create=True):
         ignored = {'D100', 'D202', 'D213'}
         errors = tuple(checker.check(['filepath'], ignore=ignored))
         error_codes = {error.code for error in errors}
         assert error_codes == expected_error_codes - ignored
 
 
+def test_skip_errors():
+    """Test that `ignore`d errors are not reported in the API."""
+    function_to_check = textwrap.dedent('''
+        def function_with_bad_docstring(foo):  # noqa: D400, D401, D403, D415
+            """ does spacinwithout a period in the end
+            no blank line after one-liner is bad. Also this - """
+            return foo
+    ''')
+    expected_error_codes = {'D100', 'D205', 'D209', 'D210', 'D213'}
+    mock_open = mock.mock_open(read_data=function_to_check)
+    from pydocstyle import checker
+    with mock.patch.object(
+            checker.tk, 'open', mock_open, create=True):
+        # Passing a blank ignore here explicitly otherwise
+        # checkers takes the pep257 ignores by default.
+        errors = tuple(checker.check(['filepath'], ignore={}))
+        error_codes = {error.code for error in errors}
+        assert error_codes == expected_error_codes
+
+    skipped_error_codes = {'D400', 'D401', 'D403', 'D415'}
+    # We need to recreate the mock, otherwise the read file is empty
+    mock_open = mock.mock_open(read_data=function_to_check)
+    with mock.patch.object(
+            checker.tk, 'open', mock_open, create=True):
+        errors = tuple(checker.check(['filepath'], ignore={},
+                                     ignore_inline_noqa=True))
+        error_codes = {error.code for error in errors}
+        assert error_codes == expected_error_codes | skipped_error_codes
+
+
 def test_run_as_named_module():
     """Test that pydocstyle can be run as a "named module".
 
@@ -196,12 +260,9 @@ def test_run_as_named_module():
         python -m pydocstyle
 
     """
-    # Running a package with "-m" is not supported in Python 2.6
-    if sys.version_info[0:2] == (2, 6):
-        return
     # Add --match='' so that no files are actually checked (to make sure that
     # the return code is 0 and to reduce execution time).
-    cmd = shlex.split("python -m pydocstyle --match=''")
+    cmd = [sys.executable, "-m", "pydocstyle", "--match=''"]
     p = subprocess.Popen(cmd,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE)
@@ -255,7 +316,7 @@ def test_sectionless_config_file(env):
         conf.write('[pdcstl]')
         config_path = conf.name
 
-    _, err, code = env.invoke('--config={}'.format(config_path))
+    _, err, code = env.invoke(f'--config={config_path}')
     assert code == 0
     assert 'Configuration file does not contain a pydocstyle section' in err
 
@@ -275,6 +336,11 @@ def test_sectionless_config_file(env):
     assert 'file does not contain a pydocstyle section' not in err
 
 
+@pytest.mark.parametrize(
+    # Don't parametrize over 'pyproject.toml'
+    # since this test applies only to '.ini' files
+    'env', ['ini'], indirect=True
+)
 def test_multiple_lined_config_file(env):
     """Test that .ini files with multi-lined entries are parsed correctly."""
     with env.open('example.py', 'wt') as example:
@@ -298,6 +364,31 @@ def test_multiple_lined_config_file(env):
     assert 'D103' not in out
 
 
+@pytest.mark.parametrize(
+    # Don't parametrize over 'tox.ini' since
+    # this test applies only to '.toml' files
+    'env', ['toml'], indirect=True
+)
+def test_accepts_select_error_code_list(env):
+    """Test that .ini files with multi-lined entries are parsed correctly."""
+    with env.open('example.py', 'wt') as example:
+        example.write(textwrap.dedent("""\
+            class Foo(object):
+                "Doc string"
+                def foo():
+                    pass
+        """))
+
+    env.write_config(select=['D100', 'D204', 'D300'])
+
+    out, err, code = env.invoke()
+    assert code == 1
+    assert 'D100' in out
+    assert 'D204' in out
+    assert 'D300' in out
+    assert 'D103' not in out
+
+
 def test_config_path(env):
     """Test that options are correctly loaded from a specific config file.
 
@@ -311,8 +402,12 @@ def test_config_path(env):
                 pass
         """))
 
+    # either my_config.ini or my_config.toml
+    config_ext = env.config_name.split('.')[-1]
+    config_name = 'my_config.' + config_ext
+
     env.write_config(ignore='D100')
-    env.write_config(name='my_config', ignore='D103')
+    env.write_config(name=config_name, ignore='D103')
 
     out, err, code = env.invoke()
     assert code == 1
@@ -320,7 +415,7 @@ def test_config_path(env):
     assert 'D103' in out
 
     out, err, code = env.invoke('--config={} -d'
-                                .format(env.get_path('my_config')))
+                                .format(env.get_path(config_name)))
     assert code == 1, out + err
     assert 'D100' in out
     assert 'D103' not in out
@@ -446,6 +541,36 @@ def test_wildcard_add_ignore_cli(env):
     assert 'D300' not in out
 
 
+@pytest.mark.parametrize(
+    # Don't parametrize over 'pyproject.toml'
+    # since this test applies only to '.ini' files
+    'env', ['ini'], indirect=True
+)
+def test_ignores_whitespace_in_fixed_option_set(env):
+    with env.open('example.py', 'wt') as example:
+        example.write("class Foo(object):\n    'Doc string'")
+    env.write_config(ignore="D100,\n  # comment\n  D300")
+    out, err, code = env.invoke()
+    assert code == 1
+    assert 'D300' not in out
+    assert err == ''
+
+
+@pytest.mark.parametrize(
+    # Don't parametrize over 'tox.ini' since
+    # this test applies only to '.toml' files
+    'env', ['toml'], indirect=True
+)
+def test_accepts_ignore_error_code_list(env):
+    with env.open('example.py', 'wt') as example:
+        example.write("class Foo(object):\n    'Doc string'")
+    env.write_config(ignore=['D100', 'D300'])
+    out, err, code = env.invoke()
+    assert code == 1
+    assert 'D300' not in out
+    assert err == ''
+
+
 def test_bad_wildcard_add_ignore_cli(env):
     """Test adding a non-existent error codes with --add-ignore."""
     with env.open('example.py', 'wt') as example:
@@ -466,6 +591,189 @@ def test_bad_wildcard_add_ignore_cli(env):
             in err)
 
 
+def test_overload_function(env):
+    """Functions decorated with @overload trigger D418 error."""
+    with env.open('example.py', 'wt') as example:
+        example.write(textwrap.dedent('''\
+        from typing import overload
+
+
+        @overload
+        def overloaded_func(a: int) -> str:
+            ...
+
+
+        @overload
+        def overloaded_func(a: str) -> str:
+            """Foo bar documentation."""
+            ...
+
+
+        def overloaded_func(a):
+            """Foo bar documentation."""
+            return str(a)
+
+        '''))
+    env.write_config(ignore="D100")
+    out, err, code = env.invoke()
+    assert code == 1
+    assert 'D418' in out
+    assert 'D103' not in out
+
+
+def test_overload_method(env):
+    """Methods decorated with @overload trigger D418 error."""
+    with env.open('example.py', 'wt') as example:
+        example.write(textwrap.dedent('''\
+        from typing import overload
+
+        class ClassWithMethods:
+            @overload
+            def overloaded_method(a: int) -> str:
+                ...
+
+
+            @overload
+            def overloaded_method(a: str) -> str:
+                """Foo bar documentation."""
+                ...
+
+
+            def overloaded_method(a):
+                """Foo bar documentation."""
+                return str(a)
+
+        '''))
+    env.write_config(ignore="D100")
+    out, err, code = env.invoke()
+    assert code == 1
+    assert 'D418' in out
+    assert 'D102' not in out
+    assert 'D103' not in out
+
+
+def test_overload_method_valid(env):
+    """Valid case for overload decorated Methods.
+
+    This shouldn't throw any errors.
+    """
+    with env.open('example.py', 'wt') as example:
+        example.write(textwrap.dedent('''\
+        from typing import overload
+
+        class ClassWithMethods:
+            """Valid docstring in public Class."""
+
+            @overload
+            def overloaded_method(a: int) -> str:
+                ...
+
+
+            @overload
+            def overloaded_method(a: str) -> str:
+                ...
+
+
+            def overloaded_method(a):
+                """Foo bar documentation."""
+                return str(a)
+
+        '''))
+    env.write_config(ignore="D100, D203")
+    out, err, code = env.invoke()
+    assert code == 0
+
+
+def test_overload_function_valid(env):
+    """Valid case for overload decorated functions.
+
+    This shouldn't throw any errors.
+    """
+    with env.open('example.py', 'wt') as example:
+        example.write(textwrap.dedent('''\
+        from typing import overload
+
+
+        @overload
+        def overloaded_func(a: int) -> str:
+            ...
+
+
+        @overload
+        def overloaded_func(a: str) -> str:
+            ...
+
+
+        def overloaded_func(a):
+            """Foo bar documentation."""
+            return str(a)
+
+        '''))
+    env.write_config(ignore="D100")
+    out, err, code = env.invoke()
+    assert code == 0
+
+
+def test_overload_nested_function(env):
+    """Nested functions decorated with @overload trigger D418 error."""
+    with env.open('example.py', 'wt') as example:
+        example.write(textwrap.dedent('''\
+        from typing import overload
+
+        def function_with_nesting():
+            """Valid docstring in public function."""
+            @overload
+            def overloaded_func(a: int) -> str:
+                ...
+
+
+            @overload
+            def overloaded_func(a: str) -> str:
+                """Foo bar documentation."""
+                ...
+
+
+            def overloaded_func(a):
+                """Foo bar documentation."""
+                return str(a)
+            '''))
+    env.write_config(ignore="D100")
+    out, err, code = env.invoke()
+    assert code == 1
+    assert 'D418' in out
+    assert 'D103' not in out
+
+
+def test_overload_nested_function_valid(env):
+    """Valid case for overload decorated nested functions.
+
+    This shouldn't throw any errors.
+    """
+    with env.open('example.py', 'wt') as example:
+        example.write(textwrap.dedent('''\
+        from typing import overload
+
+        def function_with_nesting():
+            """Adding a docstring to a function."""
+            @overload
+            def overloaded_func(a: int) -> str:
+                ...
+
+
+            @overload
+            def overloaded_func(a: str) -> str:
+                ...
+
+
+            def overloaded_func(a):
+                """Foo bar documentation."""
+                return str(a)
+            '''))
+    env.write_config(ignore="D100")
+    out, err, code = env.invoke()
+    assert code == 0
+
+
 def test_conflicting_select_ignore_config(env):
     """Test that select and ignore are mutually exclusive."""
     env.write_config(select="D100", ignore="D101")
@@ -490,30 +798,6 @@ def test_conflicting_ignore_convention_config(env):
     assert 'mutually exclusive' in err
 
 
-def test_unicode_raw(env):
-    """Test acceptance of unicode raw docstrings for python 2.x."""
-    if sys.version_info[0] >= 3:
-        return  # ur"" is a syntax error in python 3.x
-
-    # This is all to avoid a syntax error for python 3.2
-    from codecs import unicode_escape_decode
-
-    def u(x):
-        return unicode_escape_decode(x)[0]
-
-    with env.open('example.py', 'wt') as example:
-        example.write(textwrap.dedent(u('''\
-            # -*- coding: utf-8 -*-
-            def foo():
-                ur"""Check unicode: \u2611 and raw: \\\\\\\\."""
-        ''').encode('utf-8')))
-    env.write_config(ignore='D100', verbose=True)
-    out, err, code = env.invoke()
-    assert code == 0, err
-    assert 'D301' not in out
-    assert 'D302' not in out
-
-
 def test_missing_docstring_in_package(env):
     """Make sure __init__.py files are treated as packages."""
     with env.open('__init__.py', 'wt') as init:
@@ -585,6 +869,19 @@ def test_pep257_convention(env):
                 """Docstring for this class"""
                 def foo():
                     pass
+
+
+            # Original PEP-257 example from -
+            # https://www.python.org/dev/peps/pep-0257/
+            def complex(real=0.0, imag=0.0):
+                """Form a complex number.
+
+                Keyword arguments:
+                real -- the real part (default 0.0)
+                imag -- the imaginary part (default 0.0)
+                """
+                if imag == 0.0 and real == 0.0:
+                    return complex_zero
         '''))
 
     env.write_config(convention="pep257")
@@ -595,6 +892,7 @@ def test_pep257_convention(env):
     assert 'D203' not in out
     assert 'D212' not in out
     assert 'D213' not in out
+    assert 'D413' not in out
 
 
 def test_numpy_convention(env):
@@ -624,6 +922,47 @@ def test_numpy_convention(env):
     assert 'D413' not in out
 
 
+def test_google_convention(env):
+    """Test that the 'google' convention options has the correct errors."""
+    with env.open('example.py', 'wt') as example:
+        example.write(textwrap.dedent('''
+            def func(num1, num2, num_three=0):
+                """Docstring for this function.
+
+                Args:
+                    num1 (int): Number 1.
+                    num2: Number 2.
+                """
+
+
+            class Foo(object):
+                """Docstring for this class.
+
+                Attributes:
+
+                    test: Test
+
+                returns:
+                """
+                def __init__(self):
+                    pass
+        '''))
+
+    env.write_config(convention="google")
+    out, err, code = env.invoke()
+    assert code == 1
+    assert 'D107' in out
+    assert 'D213' not in out
+    assert 'D215' not in out
+    assert 'D405' in out
+    assert 'D409' not in out
+    assert 'D410' not in out
+    assert 'D412' in out
+    assert 'D413' not in out
+    assert 'D414' in out
+    assert 'D417' in out
+
+
 def test_config_file_inheritance(env):
     """Test configuration files inheritance.
 
@@ -1064,3 +1403,89 @@ def test_syntax_error_multiple_files(env):
     assert code == 1
     assert 'first.py: Cannot parse file' in err
     assert 'second.py: Cannot parse file' in err
+
+
+def test_indented_function(env):
+    """Test that nested functions do not cause IndentationError."""
+    env.write_config(ignore='D')
+    with env.open("test.py", 'wt') as fobj:
+        fobj.write(textwrap.dedent('''\
+            def foo():
+                def bar(a):
+                    """A docstring
+
+                    Args:
+                        a : An argument.
+                    """
+                    pass
+        '''))
+    out, err, code = env.invoke(args="-v")
+    assert code == 0
+    assert "IndentationError: unexpected indent" not in err
+
+
+def test_only_comment_file(env):
+    """Test that file with only comments does only cause D100."""
+    with env.open('comments.py', 'wt') as comments:
+        comments.write(
+            '#!/usr/bin/env python3\n'
+            '# -*- coding: utf-8 -*-\n'
+            '# Useless comment\n'
+            '# Just another useless comment\n'
+        )
+
+    out, _, code = env.invoke()
+    assert 'D100' in out
+    out = out.replace('D100', '')
+    for err in {'D1', 'D2', 'D3', 'D4'}:
+        assert err not in out
+    assert code == 1
+
+
+def test_comment_plus_docstring_file(env):
+    """Test that file with comments and docstring does not cause errors."""
+    with env.open('comments_plus.py', 'wt') as comments_plus:
+        comments_plus.write(
+            '#!/usr/bin/env python3\n'
+            '# -*- coding: utf-8 -*-\n'
+            '# Useless comment\n'
+            '# Just another useless comment\n'
+            '"""Module docstring."""\n'
+        )
+
+    out, _, code = env.invoke()
+    assert '' == out
+    assert code == 0
+
+
+def test_only_comment_with_noqa_file(env):
+    """Test that file with noqa and only comments does not cause errors."""
+    with env.open('comments.py', 'wt') as comments:
+        comments.write(
+            '#!/usr/bin/env python3\n'
+            '# -*- coding: utf-8 -*-\n'
+            '# Useless comment\n'
+            '# Just another useless comment\n'
+            '# noqa: D100\n'
+        )
+
+    out, _, code = env.invoke()
+    assert 'D100' not in out
+    assert code == 0
+
+
+def test_comment_with_noqa_plus_docstring_file(env):
+    """Test that file with comments, noqa, docstring does not cause errors."""
+    with env.open('comments_plus.py', 'wt') as comments_plus:
+        comments_plus.write(
+            '#!/usr/bin/env python3\n'
+            '# -*- coding: utf-8 -*-\n'
+            '# Useless comment\n'
+            '# Just another useless comment\n'
+            '# noqa: D400\n'
+            '"""Module docstring without period"""\n'
+        )
+
+    out, _, code = env.invoke()
+    assert '' == out
+    assert code == 0
